require "httparty"

module Republica2023OrLater
  class ImportJob < ApplicationJob
    queue_as :default

    def import_speakers(conference, url)
      response = HTTParty.get(url)
      if response.success?
        speakers = JSON.parse(response.body)
        speakers.each do |speaker_data|
          Speaker.find_or_initialize_by(ref_id: speaker_data["uid"], conference:).tap do |speaker|
            speaker.name = speaker_data["name_raw"]
            speaker.position = speaker_data["position"]
            speaker.description = speaker_data["bio"]
            speaker.save!
          end
        end
      else
        Rails.logger.error "Failed to fetch speakers from #{url}"
      end
    end

    def import_sessions(conference, url)
      response = HTTParty.get(url)
      if response.success?
        sessions = JSON.parse(response.body)
        sessions.reject { |s| s["langcode"] == "en" }.each do |session_data|
          stage = Stage.find_or_initialize_by(conference:, ref_id: session_data["room_nid"]).tap do |stage_|
            stage_.name = session_data["room"]
            stage_.save!
          end
          Session.find_or_initialize_by(conference:, ref_id: session_data["nid"]).tap do |session|
            session.stage = stage
            session.title = session_data["title"]
            session.language =
              case session_data["language"]
              when "German", "Deutsch"
                "de"
              when "English", "Englisch"
                "en"
              end
            session.status = session_data["status"]
            session.description = session_data["description"]
            session.session_format = session_data["format"]
            session.track = session_data["track"]
            session.is_interpreted = (session_data["live_translation"] == "1")
            session.starts_at = session_data["datetime_start"]
            session.ends_at = session_data["datetime_end"]
            session.url = "https://re-publica.com#{session_data['path']}"
            session.speakers = session_data["speaker_uid"].map { |speaker_uid| conference.speakers.find_by!(ref_id: speaker_uid) }
            session.save!
          end
        end
      else
        Rails.logger.error "Failed to fetch sessions from #{url}"
      end
    end

    def perform(conference_slug, *args)
      conference = Conference.find_by(slug: conference_slug)
      import_speakers(conference, conference.data["speakers_url"])
      import_sessions(conference, conference.data["sessions_url"])
      revision_set = RevisionSet.create!(conference:)
    end
  end
end
